#!/usr/bin/ksh

STARTTIME=`date`
# Usage of the file
if [[ $# -ne 4 ]]
then
 echo "Usage "
 echo $0 "--db=databasename --dbuser=touser --dbpassword=touserpassword --backupdir=DumpDirectory"
 echo "Example " $0 " --db=wpcdb --dbuser=wpc2 --dbpassword=ibm4wpc2 --backupdir=/u01/backup/jul10"
 exit 0
fi

# Declare the variables
DBName=`echo $1 |cut -d "=" -f2`;export DBName
typeset -u DBUSER=`echo $2 |cut -d "=" -f2`;export DBUSER
DBUserPassword=`echo $3 |cut -d "=" -f2`;export DBUserPassword
DUMPDIR=`echo $4 |cut -d "=" -f2`;export DUMPDIR
NOCREATESCHEMA=`echo $5|cut -d "=" -f2`; export NOCREATESCHEMA
cd ${DUMPDIR}

# ************************** FILE CHECKS ***************************************
if [[ -a tab9.ixf.gz ]]
then 
  gunzip *.gz
fi
if [[ ! -s db2look.sql ]] || [[ ! -s db2move.lst ]]
then
  echo "FILE NOT EXISTS"
  exit 0
fi

# Read the EXPORT.out file for any errors
if [ `grep -c SQLSTATE ${DUMPDIR}/EXPORT.out` -ne "0" ]
 then
  echo "Error encountered while exporting, Pls. verify and execute again"
  echo "The file is ${DUMPDIR}/EXPORT.out "
  exit 0;
fi
rundate=`date +%m-%d-%H:%M:%S`
cat *.msg > all_msgs_${rundate}.out

# ********************** DROP SCHEMA *******************************************
db2 "connect to ${DBName} user ${DBUSER} using ${DBUserPassword}"
echo "DROP TABLES"
db2 list tables for schema ${DBUSER} |grep '  T  ' > /tmp/tbl_list
db2 list tables for schema ${DBUSER} |grep '  A  ' > /tmp/ali_list

awk '{print "drop table '"$DBUSER"'."$1";"} ' /tmp/tbl_list > /tmp/drptbls.cmd
awk '{print "drop alias '"$DBUSER"'."$1";"} ' /tmp/ali_list > /tmp/drpali.cmd

db2 -x "select 'drop sequence " $DBUSER ".', seqname, ';'  from syscat.sequences where seqschema=upper('"$DBUSER"')" > /tmp/drpseq.cmd

db2 -tvf /tmp/drpseq.cmd > drop_${DBUSER}.log
db2 -tvf /tmp/drptbls.cmd >> drop_${DBUSER}.log
db2 -tvf /tmp/drpali.cmd >> drop_${DBUSER}.log

for e in 1 2; do
  functionlist=`db2 -x "select rtrim(funcschema)||'.'||specificname from syscat.functions where funcschema = upper('${DBUSER}')"`
  proclist=`db2 -x "select rtrim(procschema)||'.'||procname from syscat.procedures where procschema = upper('${DBUSER}')"`
  for i in $functionlist; do
    db2 -v "drop specific function ${i}" >> drop_${DBUSER}.log
  done
  for j in $proclist; do
    db2 -v "drop procedure ${j}" >> drop_${DBUSER}.log
  done
done

# ******************************************************************************
# Filter for the import only tables from db2look file
# to create only tables, indexes and functions. NO sequences will be created
sed -e '/-- This CLP file was created using/,/-- DDL statements for User Defined Functions/d' -e '/-- Database and Database Manager configuration parameters/,/-- Environment Variables settings/d'  db2look.sql  > create_schema.sql

cp  db2move.lst  db2move_${rundate}bkp.lst

exporteduser=`grep "SET CURRENT SCHEMA = " create_schema.sql | sort -u |cut -d "=" -f2 |cut -d "\"" -f2|cut -d " " -f1`;export exporteduser
#echo $exporteduser

if [ "${exporteduser}"  !=  "$DBUSER" ]
then
 # replace the username
 sed -e "s/${exporteduser}/$DBUSER/g"  create_schema.sql > create_schema_${DBUSER}.sql
 sed -e "s/${exporteduser}/$DBUSER/g"  db2move.lst >  db2move.lst_${DBUSER}.sql
 mv db2move.lst_${DBUSER}.sql db2move.lst

else
 cp create_schema.sql create_schema_${DBUSER}.sql
fi

echo "commit work;" >> create_schema_${DBUSER}.sql
echo "terminate;" >> create_schema_${DBUSER}.sql

# create only tables, indexes and functions
db2 "connect to ${DBName} user ${DBUSER} using ${DBUserPassword}"
echo "CREATE TABLES"

db2 -tvf create_schema_${DBUSER}.sql >  create_schema_${DBUSER}.log

# ******************************************************************************
# try to create table seqtab, in case it was not included in db2look output
# generated by previous versions of db2_export.sh
#db2 -v "create table seqtab (seqname varchar(30), latestval integer)" 

# Running runstats to collect statistics on the tables and indexes for optimizer
db2 "connect to ${DBName} user ${DBUSER} using ${DBUserPassword}"
echo "RUNSTATS"
db2 -x "select 'runstats on table ${DBUSER}.'||tabname||' and indexes all set profile only;' from syscat.tables where TABSCHEMA = '${DBUSER}' and type = 'T'" >runstats.sql
#cat runstats.sql|grep -i "runstats on table" > runstats_${DBUSER}.sql
db2 -tvf runstats_${DBUSER}.sql > runstats_${DBUSER}.log
db2 terminate

# Import
# ******************************************************************************
db2 "connect to ${DBName} user ${DBUSER} using ${DBUserPassword}"
echo "LOADING" 
lines=`awk 'BEGIN {ORS=""} {for (i=1; i<=NF; i++) print $i; print "\n"}' ${DUMPDIR}/db2move.lst`
for line in $lines; do
  TABLENAME=`echo $line |cut -d "!" -f2 | cut -d "." -f2| cut -d "\"" -f2`
#  echo "Loading data for table $TABLENAME"
  IXFFILE=`echo $line | cut -d "!" -f3`
  MSGFILE=`echo $line | cut -d "!" -f4`
  db2 "load from ${IXFFILE} of IXF modified by pagefreespace=0 savecount 100000 messages ${MSGFILE} replace into ${TABLENAME} statistics use profile nonrecoverable indexing mode rebuild" >> load_${DBUSER}.log
done

# ******************************************************************************
db2 -x "connect to ${DBName} user ${DBUSER} using ${DBUserPassword}"
echo "REMOVE CHECK-PENDING STATUS"
for e in 1 2; do
#  echo "iteration $e"
#  echo " Get list of tables in check pending status"
  integritylist=`db2 -x "select name from sysibm.systables where status ='C' and creator='${DBUSER}'"`
  for i in $integritylist; do
    db2 -x "set integrity for ${i} immediate checked" >> check_pending_${DBUSER}.log
  done
done
db2 terminate

# Importing the Sequences from the backup
# ******************************************************************************
db2 -x "connect to ${DBName} user ${DBUSER} using ${DBUserPassword}"
echo "CREATE SEQUENCES"
db2 -x "select 'create sequence ${DBUSER}.',seqname,' start with ',latestval,';' from seqtab" > create_sequences_${DBUSER}.sql
#cat create_sequences.sql | grep -i "create sequence ${DBUSER}" > create_sequences_${DBUSER}.sql
db2 -tvf create_sequences_${DBUSER}.sql > create_sequences_${DBUSER}.log

# If the sequences already exist - update the values
sed -e "s/create sequence/alter sequence/g" create_sequences_${DBUSER}.sql > alter_sequences.sql
sed -e "s/start with/restart with/g" alter_sequences.sql > alter_sequences_${DBUSER}.sql
db2 -tvf alter_sequences_${DBUSER}.sql > alter_sequences_${DBUSER}.log
db2 terminate

# Restoring the original files for the next import
rm create_schema.sql
rm create_schema_${DBUSER}.sql
rm create_sequences_${DBUSER}.sql
rm runstats.sql
ENDTIME=`date`
echo "start time $STARTTIME"
echo "end time $ENDTIME"
echo "CHECK .log FILES FOR SUCCESS"
exit 0
